#include "lcd/lcd.h"
#include "fatfs/tf_card.h"
#include <string.h>
#include "gd32vf103_i2c.h"
#include "minmea.h"

#define MAX_ROW 20
#define BMP180_SOLDERING_OFF_T 190
#define BMP180_SOLDERING_OFF_P 14000

#define FIXED_ALTITUDE 50

extern const char *img_sunny;
extern const char *img_rainy;
extern const char *img_stormy;
extern const char *img_fair;
extern const char *img_cloudy;

#ifdef USE_UART0
void init_uart0(void)
{	
	/* enable GPIO clock */
    rcu_periph_clock_enable(RCU_GPIOA);
    /* enable USART clock */
    rcu_periph_clock_enable(RCU_USART0);

    /* connect port to USARTx_Tx */
    gpio_init(GPIOA, GPIO_MODE_AF_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_9);
    /* connect port to USARTx_Rx */
    gpio_init(GPIOA, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_50MHZ, GPIO_PIN_10);

	/* USART configure */
    usart_deinit(USART0);
    usart_baudrate_set(USART0, 115200U);
    usart_word_length_set(USART0, USART_WL_8BIT);
    usart_stop_bit_set(USART0, USART_STB_1BIT);
    usart_parity_config(USART0, USART_PM_NONE);
    usart_hardware_flow_rts_config(USART0, USART_RTS_DISABLE);
    usart_hardware_flow_cts_config(USART0, USART_CTS_DISABLE);
    usart_receive_config(USART0, USART_RECEIVE_ENABLE);
    usart_transmit_config(USART0, USART_TRANSMIT_ENABLE);
    usart_enable(USART0);

    usart_interrupt_enable(USART0, USART_INT_RBNE);
}
#endif

#ifdef USE_GPS

#define GPS_INTERVAL 3600

void init_uart1(void)
{
	/* enable GPIO clock */
    rcu_periph_clock_enable(RCU_GPIOA);
    /* enable USART clock */
    rcu_periph_clock_enable(RCU_USART1);

    /* connect port to USARTx_Tx */
    /* gpio_init(GPIOA, GPIO_MODE_AF_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_9); */
    /* connect port to USARTx_Rx */
    gpio_init(GPIOA, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_50MHZ, GPIO_PIN_3);

	/* USART configure */
    usart_deinit(USART1);
    usart_baudrate_set(USART1, 9600);
    usart_word_length_set(USART1, USART_WL_8BIT);
    usart_stop_bit_set(USART1, USART_STB_1BIT);
    usart_parity_config(USART1, USART_PM_NONE);
    usart_hardware_flow_rts_config(USART1, USART_RTS_DISABLE);
    usart_hardware_flow_cts_config(USART1, USART_CTS_DISABLE);
    usart_receive_config(USART1, USART_RECEIVE_ENABLE);
    usart_transmit_config(USART1, USART_TRANSMIT_DISABLE);
    usart_enable(USART1);

    usart_interrupt_enable(USART1, USART_INT_RBNE);
}

#define MAX_NMEA 192
#endif

static inline void i2c_wait_busy(void)
{
    while(i2c_flag_get(I2C0, I2C_FLAG_I2CBSY))
            ;
}

static void init_i2c0(void)
{
    rcu_periph_clock_enable(RCU_GPIOB);
    rcu_periph_clock_enable(RCU_I2C0);
    gpio_init(GPIOB, GPIO_MODE_AF_PP, GPIO_OSPEED_50MHZ, GPIO_PIN_6 | GPIO_PIN_7);
    //gpio_pin_remap_config(GPIO_I2C0_REMAP, ENABLE);
    i2c_deinit(I2C0);
    i2c_clock_config(I2C0, 100000, I2C_DTCY_2);
    i2c_mode_addr_config(I2C0, I2C_I2CMODE_ENABLE, I2C_ADDFORMAT_7BITS, 0);
    i2c_enable(I2C0);
    i2c_ack_config(I2C0, I2C_ACK_ENABLE);
}


#define BMP180_I2C_ADDR 0xEE

/* Oversapling setting (0 = Ultra low pow, 3 = Ultra high res) */
#define BMP180_OSS_ULP 0
#define BMP180_OSS_STD 1
#define BMP180_OSS_HR  2
#define BMP180_OSS_UHR 3

#define BMP180_REG_T        0x2E
static const uint32_t BMP180_OSS = BMP180_OSS_UHR;
static const uint8_t BMP180_REG_P_ARRAY[4] = {0x34, 0x74, 0xB4, 0xF4};
static const uint8_t BMP180_REG_P_TIME[4] = {5, 8, 14, 26};
#define BMP180_REG_P BMP180_REG_P_ARRAY[BMP180_OSS]
#define BMP180_P_CONVERSION_TIME BMP180_REG_P_TIME[BMP180_OSS]

struct __attribute__((packed)) bmp180_cal_regs {
    union {
        struct __attribute__((packed)) creg_s {
            /* Read: 22 B */
            int16_t AC1, AC2, AC3;
            uint16_t AC4, AC5, AC6;
            int16_t B1, B2, MB, MC, MD;
        } creg;
        uint16_t reg_ptr[11];
    };
    /* Calculated params */
    int32_t B3, B5, B6;
    uint32_t B4, B7;
};

static struct bmp180_cal_regs BMP180;

/* Temperature [in 0.1 °C] */
static int32_t compensate_temperature(int16_t ut)
{
    int32_t x1 = ((ut - BMP180.creg.AC6) * BMP180.creg.AC5) >> 15;
    int32_t x2 = (BMP180.creg.MC << 11) / (x1 + BMP180.creg.MD);
    BMP180.B5 = x1 + x2;
    return (BMP180.B5 + 8) >> 4;
}

/* Pressure [in Pa, 1 Pa = 0.01mbar] */
static uint32_t compensate_pressure(uint32_t up)
{
    uint64_t x1, x2, x3;
    uint32_t p;
    BMP180.B6 = BMP180.B5 - 4000;
    x1 = (BMP180.creg.B2 * ((BMP180.B6 * BMP180.B6) >> 12)) >> 11;
    x2 = (BMP180.creg.AC2 * BMP180.B6) >> 11;
    x3 = x1 + x2;
    BMP180.B3 = ((((int64_t)BMP180.creg.AC1 * 4 + x3) << BMP180_OSS) + 2) >> 2;
    x1 = (BMP180.creg.AC3 * BMP180.B6) >> 13;
    x2 = (BMP180.creg.B1 * ((BMP180.B6 * BMP180.B6) >> 12)) >> 16;
    x3 = ((x1 + x2) + 2)  >> 2;
    BMP180.B4 = (BMP180.creg.AC4 * (x3 + 32768UL)) >> 15;
    BMP180.B7 = (up - BMP180.B3) * (50000UL >> BMP180_OSS);
    if (BMP180.B7 < 0x80000000UL)
        p = (BMP180.B7 * 2) / BMP180.B4;
    else
        p = (BMP180.B7 / BMP180.B4) * 2;
    x1 = (p >> 8) * (p >> 8);
    x1 = (x1 * 3038UL) >> 16;
    x2 = (-7357UL * p) >> 16;
    p += (x1 + x2 + 3791UL) >> 4;
    return p;
}

static void bmp180_calibrate(void)
{
    uint8_t reg_msb, reg_lsb;
    uint8_t r_start = 0xAA;
    unsigned i;
    i2c_wait_busy();
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, 0xAA);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_RECEIVER);
    i2c_ack_config(I2C0, I2C_ACK_ENABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    for (i = 0; i < 10; i++) {
        while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
            ;
        reg_msb = i2c_data_receive(I2C0);
        while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
            ;
        reg_lsb = i2c_data_receive(I2C0);
        BMP180.reg_ptr[i] = (reg_msb << 8) + reg_lsb;
    }
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    reg_msb = i2c_data_receive(I2C0);
    while(!i2c_flag_get(I2C0, I2C_FLAG_BTC))
        ;
    i2c_ack_config(I2C0, I2C_ACK_DISABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    reg_lsb = i2c_data_receive(I2C0);
    i2c_stop_on_bus(I2C0);
    while (I2C_CTL0(I2C0) & 0x0200)
        ;
    BMP180.reg_ptr[10] = (reg_msb << 8) + reg_lsb;
}

static int16_t bmp180_read_T(void)
{
    uint8_t data_msb, data_lsb;
    i2c_wait_busy();
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, 0xF4);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, BMP180_REG_T);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_stop_on_bus(I2C0);
    while (I2C_CTL0(I2C0) & 0x0200)
        ;

    delay_1ms(5);
    i2c_wait_busy();
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, 0xF6);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_RECEIVER);
    i2c_ack_config(I2C0, I2C_ACK_ENABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    data_msb = i2c_data_receive(I2C0);
    while(!i2c_flag_get(I2C0, I2C_FLAG_BTC))
        ;
    i2c_ack_config(I2C0, I2C_ACK_DISABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    data_lsb = i2c_data_receive(I2C0);
    i2c_stop_on_bus(I2C0);
    while (I2C_CTL0(I2C0) & 0x0200)
        ;
    return (int16_t)(data_lsb + (data_msb << 8));
}

static uint32_t bmp180_read_P(void)
{
    uint8_t data_msb, data_lsb, data_xlsb;
    i2c_wait_busy();
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, 0xF4);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, BMP180_REG_P);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_stop_on_bus(I2C0);
    while (I2C_CTL0(I2C0) & 0x0200)
        ;

    delay_1ms(BMP180_P_CONVERSION_TIME);
    i2c_wait_busy();
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_data_transmit(I2C0, 0xF6);
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE))
        ;
    i2c_start_on_bus(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND))
        ;
    i2c_master_addressing(I2C0, BMP180_I2C_ADDR, I2C_RECEIVER);
    i2c_ack_config(I2C0, I2C_ACK_ENABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND))
        ;
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    data_msb = i2c_data_receive(I2C0);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    data_lsb = i2c_data_receive(I2C0);
    while(!i2c_flag_get(I2C0, I2C_FLAG_BTC))
        ;
    i2c_ack_config(I2C0, I2C_ACK_DISABLE);
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE))
        ;
    data_xlsb = i2c_data_receive(I2C0);
    i2c_stop_on_bus(I2C0);
    while (I2C_CTL0(I2C0) & 0x0200)
        ;
    return (uint32_t)((data_xlsb + (data_lsb << 8) + (data_msb << 16)) >> (8 - BMP180_OSS));
}

static int64_t time_offset = 0;
static uint32_t get_today_time(void)
{
    uint64_t machine_time = get_timer_value() / (SystemCoreClock >> 2);
    return (uint32_t)((machine_time % (24 * 3600)) + time_offset);
}

static void set_today_time(int64_t seconds_since_midnight)
{
    uint64_t machine_time = get_timer_value() / (SystemCoreClock >> 2);
    seconds_since_midnight -= machine_time % (24 * 3600);
    time_offset = seconds_since_midnight;
}



#ifdef USE_GPS
/* UART/GPS */

static struct minmea_sentence_rmc global_minmea_gps;
static struct minmea_sentence_gga global_minmea_gga;

static char uart_data_receive_blocking(uint32_t usart_periph)
{
    while (usart_flag_get(usart_periph, USART_FLAG_RBNE) != SET) {
            continue;
    }
    return usart_data_receive(usart_periph);
}

static void get_nmea_line(char *buf)
{
    int i = 0;
    while (uart_data_receive_blocking(USART1) != '$') {
        /* sync with the beginning of the line */
    }
    buf[i++] = '$';
    while(true) {
        buf[i] = uart_data_receive_blocking(USART1);
        if (buf[i] == '\n' || i >= MAX_NMEA)
            break;
        i++;
    }
    buf[i] = 0;
}

static int read_gps(void)
{
    char nmea_buf[MAX_NMEA];
    int ret;
    int gga = 0, rmc = 0;
    int nmea_id;
    do {
        get_nmea_line(nmea_buf);
        nmea_id = minmea_sentence_id(nmea_buf, 0);
        switch(nmea_id) {
            case MINMEA_SENTENCE_RMC:
                ret = minmea_parse_rmc(&global_minmea_gps, nmea_buf);
                rmc++;
#ifdef DEBUG_NMEA
                LCD_ShowString(0, 0, (u8 *)nmea_buf, YELLOW);
                delay_1ms(200);
#endif
                break;
            case MINMEA_SENTENCE_GGA:
                ret = minmea_parse_gga(&global_minmea_gga, nmea_buf);
                gga++;
#ifdef DEBUG_NMEA
                LCD_ShowString(0, 0, (u8 *)nmea_buf, MAGENTA);
                delay_1ms(200);
#endif
                break;
            default:
                ret = !0;
                break;
        }
    } while((ret != 0) && (gga == 0));
    if (!ret) {
        LCD_ShowString(0, 0, (u8 *)("PARSING ERROR"), RED);
        LCD_ShowString(0, 32, (u8 *)nmea_buf, RED);
        delay_1ms(2000);
        return -1;
    }
    return 0;
}

/* Main */
static void gps_on(void)
{
    gpio_bit_write(GPIOA, GPIO_PIN_0, 1);
}

static void gps_off(void)
{
    gpio_bit_write(GPIOA, GPIO_PIN_0, 0);
}

static int find_gps_fix(void)
{
    char gps[MAX_ROW]="GPS Data";
    uint64_t time_now;
    gps_on();
    while(!global_minmea_gps.valid) {
        if (read_gps() < 0) {
            LCD_Clear(BLACK);
            snprintf(gps, MAX_ROW-1, "Waiting for GPS");
            LCD_ShowString (8, 0, (u8 *)gps, GREEN);
        } else {
            uint32_t time_now;
            //LCD_Clear(BLACK);
            snprintf(gps, MAX_ROW-1, "awaiting GPS Fix");
            LCD_ShowString (0, 0, (u8 *)gps, GREEN);
            snprintf(gps, MAX_ROW-1,"v:%d, sat:%d, q:%d",
                    (int)global_minmea_gps.valid,
                    global_minmea_gga.satellites_tracked,
                    global_minmea_gga.fix_quality);
            if (global_minmea_gga.satellites_tracked > 0) {
                LCD_ShowString (0, 16, (u8 *)gps, CYAN);
            } else {
                LCD_ShowString (0, 16, (u8 *)gps, RED);
            }

            time_now = get_today_time();
            snprintf(gps, MAX_ROW-1, "t:%02d:%02d:%02d",
                    time_now / 3600,
                    (time_now % 3600) / 60,
                    time_now % 60);
            LCD_ShowString (0, 32, (u8 *)gps, GREEN);
            /* Got fix? */
            if (global_minmea_gps.valid) {
                /* Set time */
                if (time_offset == 0) {
                    uint64_t seconds_since_midnight = global_minmea_gga.time.seconds +
                        global_minmea_gga.time.minutes * 60 +
                        global_minmea_gga.time.hours * 3600;
                    set_today_time(seconds_since_midnight);
                }
                /* Turn off GPS */
                gps_off();
            }
        }
        show_pT();
    }
}

#endif

enum weather_prediction {
    SUNNY = 0,
    FAIR,
    CLOUDY,
    RAINY,
    STORMY,
};
    
static void show_weather(enum weather_prediction wp)
{
    if (wp == SUNNY)
        LCD_ShowPicture(img_sunny, 0, 0, 47, 47);
    if (wp == FAIR)
        LCD_ShowPicture(img_fair, 0, 0, 47, 47);
    if (wp == CLOUDY)
        LCD_ShowPicture(img_cloudy, 0, 0, 47, 47);
    if (wp == RAINY)
        LCD_ShowPicture(img_rainy, 0, 0, 47, 47);
    if (wp == STORMY)
        LCD_ShowPicture(img_stormy, 0, 0, 47, 47);
}

static uint32_t p100;
static int16_t T100;
static enum weather_prediction CW = FAIR;
static int32_t weather_delta = 0; 
static uint32_t historic_p = 0;
static int16_t historic_T = -27400;
static char weather_desc[MAX_ROW] = "Stable";
static int weather_color = GREEN;

static void show_ui(void)
{
    char prs[MAX_ROW]="Pressure";
    char tmp[MAX_ROW]="Temperature";
    char histp[MAX_ROW] = "History";
    char histT[MAX_ROW] = "History T";
    char lastrow[MAX_ROW] = "";
#if 0
    snprintf(histp, MAX_ROW - 1, "hp: %u.%02u", historic_p/100, historic_p % 100);
    LCD_ShowString(48, 0, (u8 *)(histp), MAGENTA);
    snprintf(histT, MAX_ROW - 1, "hT: %d.%d", historic_T/10, historic_T % 10);
    LCD_ShowString(48, 16, (u8 *)(histT), MAGENTA);
#endif
    snprintf(prs, MAX_ROW - 1, "%u.%02u hPa", p100/100, p100 % 100);
    LCD_ShowString(60, 20, (u8 *)(prs), CYAN);
    snprintf(tmp, MAX_ROW - 1, " %d.%d C", T100/10, T100 % 10);
    LCD_ShowString(60, 36, (u8 *)(tmp), GREEN);
    LCD_ShowString(0, 64, (u8 *)(weather_desc), weather_color);
    show_weather(CW);
}


#define SAMPLE_INTERVAL  300
#define CHANGE_THRESHOLD 100  /* in Pa */

#define HIGH_PRESSURE    102700
#define CHANGE_PRESSURE  100000
#define LOW_PRESSURE      99800



static void predict_weather(void)
{
    static enum weather_prediction historic_wp = FAIR;
    int pdiff, Tdiff;
    sprintf(weather_desc, "Stable.");


    if (historic_p == 0) {
        historic_p = p100;
        historic_T = T100;
        weather_delta = 0;
        return;
    }

    /* First switch current weather by checking high/low average */
    if (p100 > HIGH_PRESSURE) {
        sprintf(weather_desc, "Dry.");
        weather_color = YELLOW;
        CW = SUNNY;
    }
    if (p100 < LOW_PRESSURE) {
        sprintf(weather_desc, "Wet.");
        weather_color = CYAN;
        CW = RAINY;
    } else if (p100 < CHANGE_PRESSURE) {
        sprintf(weather_desc, "Low pressure");
        weather_color = CYAN;
        CW = CLOUDY;
    }

    pdiff = p100 - historic_p;
    Tdiff = T100 - historic_T;
    weather_delta = 0;


    /* p increases, T decreases, or viceversa. Take into account */
    if (((pdiff > 0) && (Tdiff < 0)) || ((pdiff < 0) && (Tdiff > 0)))
    {
        weather_delta = pdiff + ((0 - Tdiff)/10);
    } else {
        weather_delta = pdiff;
    }

    if ((weather_delta > 0) && (weather_delta > CHANGE_THRESHOLD)) {
        sprintf(weather_desc, "Improving.");
        weather_color = GREEN;
        if (CW != SUNNY) {
            CW--;
            historic_p = p100;
            historic_T = T100;
        }
    }
    else if ((weather_delta < 0) && ((0 - weather_delta) > CHANGE_THRESHOLD)) {
        sprintf(weather_desc, "Worsening.");
        weather_color = CYAN;
        if (CW != STORMY) {
            CW++;
            historic_p = p100;
            historic_T = T100;
        }
    } else {
        historic_p = (63 * historic_p + p100) / 64;
        historic_T = (63 * historic_T + T100) / 8;
    }
    if (weather_delta > 0) {
        strcat(weather_desc, " (+)");
    } else if (weather_delta < 0) {
        strcat(weather_desc, " (-)");
    } else {
        strcat(weather_desc, " (=)");
    }
}

static void sample_pT(void)
{
    int16_t ut;
    uint32_t up;
    uint32_t pressure;
    int32_t temperature;
    ut = bmp180_read_T();
    up = bmp180_read_P();

    /* Device offset */
    ut -= BMP180_SOLDERING_OFF_T;
    up -= BMP180_SOLDERING_OFF_P;

    temperature = compensate_temperature(ut);
    pressure = compensate_pressure(up);

    /* Normalize to sea level */
    p100 = pressure + FIXED_ALTITUDE * 12;
    T100 = temperature;
    predict_weather();

}


int main(void)
{
    uint8_t mount_is_ok = 1; /* 0: mount successful ; 1: mount failed */
    uint32_t s_count = 0;
    int display_on;
    uint32_t display_on_count = 0;
    uint32_t display_off_count = 10;
    const uint32_t sample_interval = 30;

    SystemInit();
    /* Init GPIOs */
    rcu_periph_clock_enable(RCU_GPIOA);
    rcu_periph_clock_enable(RCU_GPIOC);
    LEDR(0);

    /* User Button */
    gpio_init(GPIOA, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_2MHZ, GPIO_PIN_8);

#ifdef USE_GPS
    /* GPS on/off */
    gpio_init(GPIOA, GPIO_MODE_OUT_PP, GPIO_OSPEED_2MHZ, GPIO_PIN_0);
    gps_off();
#endif
    
    /* User Button */
    gpio_init(GPIOA, GPIO_MODE_IN_FLOATING, GPIO_OSPEED_2MHZ, GPIO_PIN_8);

    /* HW init: uart, ADC, Display */
#ifdef USE_UART0
    init_uart0();
#endif
#ifdef USE_GPS
    init_uart1();
#endif
    Lcd_Init();

    /* I2C init */
    init_i2c0();

    /* Read calibration registers */
    bmp180_calibrate();


#ifdef USE_GPS
    /* Wait for GPS fix */
    find_gps_fix();
#endif
    sample_pT();
	LCD_WR_REG(0x10);	// Display Off
    OLED_BLK_Clr();
    LCD_Clear(BLACK);

    /* Main loop */
    while (1)
    {
        delay_1ms(1000);
        if (gpio_input_bit_get(GPIOA, GPIO_PIN_8)) {
	        LCD_WR_REG(0x11);	// Display On
            OLED_BLK_Set();
            display_on = 1;
            display_on_count = 0;
            gpio_bit_reset(GPIOB, GPIO_PIN_0);
        }
        if (display_on) {
            show_ui();
            if (display_on_count++ > display_off_count) {
                LCD_Clear(BLACK);
	            LCD_WR_REG(0x10);	// Display Off
                OLED_BLK_Clr();
                display_on = 0;
                gpio_bit_set(GPIOB, GPIO_PIN_0);
            }
        }
        if(s_count++ >= SAMPLE_INTERVAL) {
            s_count = 0;
            sample_pT();
        }
    }
}

#ifdef USE_UART0
int _put_char(int ch)
{
    usart_data_transmit(USART0, (uint8_t) ch );
    while ( usart_flag_get(USART0, USART_FLAG_TBE)== RESET){
    }

    return ch;
}
#endif
